# The lines from here to the =cut are part of Perl's
# internal documentation system.  If you want 
# view the documentation use the command:
#
#	pod2text <script>
#
=pod

=head1 NAME

time_over - Package to add/subtract times (with operator overloading)

=head1 SYNOPSIS

    use time_over;

    my $time = time_over->new(1, 30);

    $time1 = $time2 + time3;

    $time1 = $time2 - time3;

    print "Time is $time1\n";

=head1 DESCRIPTION

The I<time_over> package handles the addition, subtraction
and printing of times.  This is done through operator overloading.

=head1 FUNCTIONS

=over 4

=item I<$time = >B<time_over->new(>I<$hours, [$minutes]>B<);>

Creates a new time.

=item I<$time1 >B<=>I< $time2 >B<+>I< time3;>

Adds two times.

=item I<$time1 >B<=>I< $time2 >B<->I< time3;

Subtract two times.

=item I<print "Time is $time1\n";>

Turns a time into a string for assignment or printing.

=back

=head1 AUTHOR

Steve Oualline, E<lt>oualline@www.oualline.comE<gt>.

=head1 COPYRIGHT

Copyright 2002 Steve Oualline.
This program is distributed under the GPL.  

=cut
use strict;
use warnings;
package time_over;

# Prototypes for the functions to use 
sub add($$);
sub subtract($$$);
sub to_string($);

# List the functions we are overloading
use overload (
    '+' => \&add,
    '-' => \&subtract,
    '""' => \&to_string
);

################################################################
# my $time = new($hours, [$minutes])
#
# Create a new time.
################################################################
sub new($$;$)
{
    my $class = shift;   # The class name
    my $hours = shift;   # Initial hours
    my $minutes = shift; # Initial minutes
    my $self = {};       # The hash we are creating

    if (not defined($minutes)) {
	$minutes = 0;
    }

    $self->{hours} = $hours;
    $self->{minutes} = $minutes;
    return (bless $self, $class);
}

################################################################
# $time1 = $time2 + $time3 
#
# Add two times
################################################################
sub add($$)
{
    my $time1 = shift;   # The first time for adding
    my $time2 = shift;   # The second time for adding
    my $result = {};     # Result of the addition

    if (not ref($time2)) {
        $time2 = new time_over($time2);
    }

    $result->{hours} = $time1->{hours} + $time2->{hours};
    $result->{minutes} = $time1->{minutes} + $time2->{minutes};
    $result->{hours} += int($result->{minutes} / 60);
    $result->{minutes} %= 60;
    return (bless $result);
}

################################################################
# $time1 = $time2 - $time3 
#
# Subtract two times
################################################################
sub subtract($$$)
{
    my $time1 = shift;   # The hash we wish to subtract from
    my $time2 = shift;   # The other hash
    my $flag = shift;    # The manipulation flag
    my $result = {};     # The result we are computing

    if (not ref($time2)) {
        $time2 = new time_over($time2);
    }
    if ($flag) {
        ($time1, $time2) = ($time2, $time1);
    }
    $result->{hours} = $time1->{hours} - $time2->{hours};
    $result->{minutes} = $time1->{minutes} - $time2->{minutes};
    while ($result->{minutes} < 0) {
        $result->{minutes} += 60;
        $result->{hours}--;
    }
    return (bless ($result));
}

################################################################
# to_string -- Turn a time into a string (whenever Perl thinks
# we need a string version of a time.)
################################################################
sub to_string($)
{
    my $time = shift;     # The hash to turn into a string

    return(sprintf("%d:%02d", 
	$time->{hours}, $time->{minutes}));
}

1;
