# The lines from here to the =cut are part of Perl's
# internal documentation system.  If you want 
# view the documentation use the command:
#
#	pod2text <script>
#
=pod

=head1 NAME

tk_grep.pl - Visual version of a grep program

=head1 SYNOPSIS

    perl tk_grep.pl

=head1 DESCRIPTION

The I<tk_grep.pl> provides the user with a GUI which allows you
to search a series of files for a pattern.

=head1 AUTHOR

Steve Oualline, E<lt>oualline@www.oualline.comE<gt>.

=head1 COPYRIGHT

Copyright 2002 Steve Oualline.
This program is distributed under the GPL.  

=cut
use strict;
use warnings;

use Tk;
use Tk::LabEntry;

my $pattern = "";       # The pattern to search for
my $file_list = "*.c";  # The files to search

######################################################
# error_box($parent, $message)
#
# Display an error message and wait until the user
# presses OK to continue.
######################################################
sub error_box($$)
{
    my $parent = shift; # The parent window
    my $message = shift;# The message to dispaly

    $parent->messageBox(
        -icon => "error",
        -title => "Error",
        -message => $message,
        -type => "ok");
}
######################################################
# search($main_window)
#
# Perform a search through all the files 
######################################################
sub search($)
{
    my $parent = shift; # The parent window for our popup

    if ($pattern eq "") {
        error_box($parent, "No search pattern");
        return;
    }
    if ($file_list eq "") {
        error_box($parent, "No file specification");
        return;
    }

    # The list of files to search
    my @file_list = glob($file_list);
    if ($#file_list == -1) {
        error_box($parent, "No files match the specification");
        return;
    }

    # The top level window for our results
    my $tk_top = $parent->Toplevel();

    # The window to contains the actual text of the 
    # results
    my $tk_text = $tk_top->Scrolled(
            'Text',
            -scrollbars => "osoe",
            -width => 80,
            -height => 10,
            -wrap => 'none'
        )->pack(
            -side => "top",
            -expand => 1,
            -fill => "both"
        );

    $tk_top->Button(
            -text => "Close",
            -command => sub {$tk_top->destroy();}
        )->pack(
            -side => "top",
            -anchor => "w"
        );

    # Loop through each file
    foreach my $cur_file (@file_list)
    {
        if (not open(IN_FILE, "<$cur_file"))
        {
            error_box($parent, "Could not open $cur_file");
            next;
        }

        # Loop through each line
        my $line_number = 1;
        while (1) {
            my $line = <IN_FILE>;

            if (not defined($line)) {
                last;
            }
                
            if ($line =~ $pattern) {
                my $display_line = sprintf("%3d %s", 
                        $line_number, $line);
                $tk_text->insert('end', $display_line);
            }
            $line_number++;
        }
        close (IN_FILE);
    }
}


######################################################
# lab_box($window, $label, $var)
#
# Create a box consisting of a label and a entry.
######################################################
sub lab_box($$$)
{
    my $window = shift; # the parent of the widgets
    my $label = shift;  # The text for the label
    my $var = shift;    # The variable for the entry

    # The frame we use to hold our two widgets
    my $frame = $window->Frame(
        )->pack(
            -side => "top",
            -fill => "x"
        );
    $frame->Label(
            -text => $label
        )->pack(
            -side => "left"
        );
    # The entry widget
    my $tk_entry = $frame->Entry(
            -textvariable => $var
        )->pack(
            -side => "right",
            -fill => "x",
            -expand => 1
        );
    $tk_entry->bind('<Return>', sub {search($window);});
}

# The main window
my $tk_mw = MainWindow->new();
lab_box($tk_mw, "Pattern: ", \$pattern);
lab_box($tk_mw, "Files: ", \$file_list);

# The frame we use for the buttons
my $button_frame = $tk_mw->Frame(
    )->pack( 
        -side => "top",
        -fill => "both"
    );

$button_frame->Button(
        -text => "Search",
        -command => sub {search($tk_mw);}
    )->pack(
        -side => "left",
        -anchor => "e"
    );

$button_frame->Button(
        -text => "Cancel",
        -command => sub {print "Cancel Pushed\n"; exit(); }
    )->pack(
        -side => "right",
        -anchor => "w"
    );

MainLoop(); 
