use strict;
use warnings;

package disks;
=pod

=head1 NAME

disks.pm - Data database manipulation class

=head1 SYNOPSIS

    $db = disks->new($file, $read_only)

    $db->update_disk($id, %data);

    $id = $db->new_disk($category, %data);

    $disk_ref = $db->get_disk($id);

    @keys = $db->get_keys();

    delete_disk($id)
    
=head1 DESCRIPTION

The I<disks> package provides functions that allow the user to
store and retrieve information about a set of disks.  The disks
can be floppy, zip, cd-rom, or whatever.  The database makes no
distinction between them.

=head1 Functions

=over 4

=item $db = disk->new($file, $read_only)

Open a database. 

=head2 Parameters 

=item I<$file> 

The file name for the database.

=item I<$read_only>

If set, the database is opened read only.  If not set, the database
is opened for reading and writing.

=back

The database will be created if does not exist (and you are opening
it read/write.)

=head2 Return

The database object or undef it the database could not
be opened.

=item $db->update_disk($id, %data);

Update the information about a given disk in the database.

=head2 Parameters

=over 4

=item I<$id>

Id of the disk.  

=item I<%data>

Hash containing the information about the disk.

=back

=head2 Return Value

None.

=item $id = $db->new_disk($category, %data);

Returns the entire set of data store in the database.  This is a 
hash reference who's keys are the days and who's values are a
hash containing the key items listed above.

=head2 Parameters

=over 4

=item $category

The category letter for the new data.  The system will use this
to find a slot in the database for this disk.

=item %data

The data describing the disk.

=back

=head2 Returns

The generated ID number of the disk.

=item $disk_ref = $db->get_disk($id);

Get the disk information for a given disk.

=head2 Parameters

=over 4 

=item $id

The id of the disk we want.

=back

=head2 Returns

A reference to the hash containing the disks data or
undef if the id does not match any disk.

=item @keys = $db->get_keys();

Returns a lit of all the keys in the database.

=item delete_disk($id)

Deletes the disk from the database

=back

=cut

use Fcntl;
use DB_File;
use MLDBM qw(DB_File Storable);
use Carp;
use POSIX;

########################################################
# new($db_file, $flag) -- Open a disk database 
#
#  If $flag is set, enable writing.
########################################################
sub new($$) {
    my $class = shift;  # The class name
    my $db_file = shift;# The name of the database file
    my $flag = shift;   # Read only flag
    my $self = {};      # Hash that represents me

    my $database;       # The database object 
    my %disk_db;        # The disk data in the database

    if ($flag) {
        # The database information
        $database = tie (%disk_db, 
                'MLDBM', $db_file,
                O_RDONLY, 0640);
        $self->{READ_ONLY} = 1;
    } else {
        # The database information
        $database = tie (%disk_db, 
                'MLDBM', $db_file,
                O_RDWR|O_CREAT, 0640);
        $self->{READ_ONLY} = 0;
    }
    if (not defined($database)) {
        return (undef);
    }
    $self->{DATABASE} = $database;
    $self->{DISK_DATA} = \%disk_db;
    return (bless($self, $class));
}
#######################################################
# update_disk($id, %data)
# 
# Update the data for a disk
#######################################################
sub update_disk($$%) {
    my $self = shift;           # The class
    my $id = shift;             # The disk id number
    my %data = @_;              # The data to record

    if ($self->{READ_ONLY}) {
        die("Database is read only");
    }
    $self->{DISK_DATA}->{$id} = {%data};
}
#######################################################
# new_disk($category, %data)
# 
# Create a new disk
#######################################################
sub new_disk($$%) {
    my $self = shift;           # The class
    my $category = shift;       # The disk category
    my %data = @_;              # The data to record

    if ($self->{READ_ONLY}) {
        die("Database is read only");
    }
    my $index;  # Index used for computing the id
    my $id;     # The new id number we just generated

    for ($index = 0; ; $index++) {
        $id = $category.$index;
        if (not defined($self->{DISK_DATA}->{$id})) {
            last;
        }
    }
    $self->{DISK_DATA}->{$id} = {%data};
    return ($id);
}
#######################################################
# get_disk -- Returns a hash containing disk information
#       on a given disk
#
# Returns a reference to the data hash
#######################################################
sub get_disk($) {
    my $self = shift;   # The class
    my $id = shift;     # The id for the disk

    return ($self->{DISK_DATA}->{$id});
}
#######################################################
# delete_disk -- Delete a given disk
#######################################################
sub delete_disk($$) {
    my $self = shift;   # The class
    my $id = shift;     # The id for the disk

    delete ($self->{DISK_DATA}->{$id});
}
#######################################################
# key_sort -- Sort by the disk key (<letter><number>)
#
# Keys are in $a and $b, returns number as required by 
# sort.
#######################################################
sub key_sort {
    $a =~ /(.)(.*)/;
    my $letter1 = $1;	# Letters in the first key
    my $digits1 = $2;	# Digits in the first key

    $b =~ /(.)(.*)/;
    my $letter2 = $1;	# Letters in the second key
    my $digits2 = $2;	# Digits in the second key

    if ($letter1 ne $letter2) {
        return ($letter1 cmp $letter2);
    }
    return ($digits1 <=> $digits2);
}
#######################################################
# get_keys -- Return a list of all the keys in the
#       database
#######################################################
sub get_keys($)
{
    my $self = shift;   # The class

    return (sort key_sort keys %{$self->{DISK_DATA}});
}

#######################################################
# destroy the variable (called automatically by perl)
#######################################################
sub DESTROY($)
{
    my $self = shift;   # The class
    untie $self->{DISK_DATA};
}

1;
