=pod

=head1 NAME

cal.pl - Check a calendar for things coming up and things past

=head1 SYNOPSIS

    cal.pl <calendar-file>

=head1 DESCRIPTION

The I<cal.pl> checks the calendar file to see if anything is coming
up soon.  It also reminds you of past envents and how long they
happened.  

The format of the calendar file is:

I<date> [I<flags>] I<information>

The <date> is a date in any format so long as there are no spaces in
in it.   

The I<flags> are:

=over 4

=item +I<days>

Remind me of an event that's coming up in I<days> days or less.

=item -

Tell me about an event that's passed.  (This is so I can see how
long it's been sent I sent out that rebate request.)

=back

The I<information> field is the text that describes the event.

=head1 EXAMPLES

I sent the rebate out on March 30.  How many days ago was that?

    3/30 - Epson Rebate

Remind me two weeks before about my dentist visit on June 1.

    1-June-2002 +14 Dentist

=cut
use strict;
use warnings;
use Time::ParseDate;

if ($#ARGV != 0) {
    print "Usage is $0 <calendar file>\n";
}

# The number of seconds per day
my $SECONDS_PER_DAY = 60 * 60 * 24;

open CAL_FILE, "<$ARGV[0]" or
    die("Could not open $ARGV[0]");

# Todays date (in system format)
my $today = time();

while (1) {
    # The line from the input file
    my $line = <CAL_FILE>;
    if (not defined($line)) {
	last;
    }
    chomp($line);
    if ($line eq "") {
        next;
    }

    # Decode the line 
    #	<spaces>(date-$1)<spaces>(rest-$2)
    $line =~ /^\s*(\S+)\s(.*)/;
    my $date_str = $1;	# Get the date
    my $rest = $2;	# Get the rest of the line
    my $flag;		# Flag information

    # We might have a line like:
    #    ([+-]digits-$1)<spaces>($rest-$2)
    if ($rest =~ /^([+-]\S*)\s*(.*)$/) {
	$flag = $1;	# Grab the flag

	# Put in a default number of dayes if needed
	if ($flag eq "-") {
	    $flag = -9999;
	}

	$rest = $2;	# Grab the rest
    } else {
	$flag = 7;	# Assume the flag is +7
    }

    my $date = parsedate($date_str);
    if (not defined($date)) {
	print STDERR "Unable to parse date: $date_str\n";
	next;
    }

    # Compute difference between today and the date given
    my $diff = int (($date - $today) / $SECONDS_PER_DAY);

    if ($flag > 0) {
	if ($diff < 0) {
	    next;
	}
	if ($diff < $flag) {
	    print "$date_str ($diff) $rest\n";
	}
    } else {
	# Is the event so old it's in the future
	if ($diff > 0) {
	    $diff -= 365;
	}
	if ($diff > $flag) {
	    print "$date_str ($diff) $rest\n";
	}
    }
}
