=pod

=head1 NAME

log_file - View log files.

=head1 SYNOPSIS

B<log_file> 
[B<-c> I<config_file>]
I<log_file>

=head1 DESCRIPTION

The I<log_file> reads a log file and displays it in a window.
Lines containing the word "Error" are highlighted.

The button "Next Error" jumps to the next line containing an
error.  "Previous Error" goes up one.   "First", "Last" go to
the first / last error.

The "Ignore" button causes a line to be ignored.  It will not be
considered an error.   

=head1 OPTIONS

=over 4

=item -c I<config_file>

Specify a configuration file to use which contains a definition
of the errors and the ignore strings.   The format of this file
is:

	ERRORS
	<error regular expression>
	<error regular expression>
	IGNORE
	<ignore regular expression>
	<ignore regular expression>
	<ignore regular expression>

If none is specifed, the default "$HOME/.log_file" is used.

=back

=cut

use strict;
use warnings;
use Getopt::Std;
use Tk;

# The strings that indicate we have an error (default value)
my @error_check = ('\bError\b', '\berror\b');

# Location list (where the errors are)
my @error_loc = ();	

my $loc_index = 0;	# Index into the error location list

my @ignore_list = ();	# List of lines to ignore

my $main_window;	# Top window
my $text_box;		# Widget containg the error information
my $count_label;	# label contianing the error count

my @file_lines = ();	# The lines in the file

my $config_change = 0;	# True if we changed the configuration

use vars qw($opt_c);

########################################################
# Tell user what to do
########################################################
sub usage()
{
    print STDERR "Usage is: $0 [-c <config>] <log-file>\n";
    exit (8);
}
########################################################
# show_loc -- Make sure that the current error location is
# visible on the screen.
########################################################
sub show_loc()
{
    # Make sure that $loc_index in in range
    if ($loc_index < 0)
    {
	$loc_index = 0;
    }
    if ($loc_index > $#error_loc)
    {
	$loc_index  = $#error_loc;
    }
    if ($#error_loc < 0)
    {
	return;
    }
    $text_box->yview(-pickplace, "$error_loc[$loc_index].0");
}
########################################################
# find_errors -- Locate the lines within the text which contain
# errors
########################################################
sub find_errors()
{
    @error_loc = ();	# Zero the error location list

    my $old_cursor = $main_window->cget(-cursor);
    $main_window->configure(-cursor => "watch");
    # The line number of the current line.
LINE_LOOP:
    for (my $line_number = 0; 
    	$line_number <= $#file_lines; 
	$line_number++)
    {
        # The line we are reading
	my $line = $file_lines[$line_number];
	foreach my $cur_ignore (@ignore_list)
	{
	    if ($line =~ /$cur_ignore/)
	    {
		next LINE_LOOP;
	    }
	}
	foreach my $cur_error (@error_check)
	{
	    if ($line =~ /$cur_error/)
	    {
		# Text box uses 1 based number/Perl uses 0
		my $text_line = $line_number + 1;
		$text_box->tagAdd(
			"highlight",
			"$text_line.0", 
			"$text_line.0 lineend"
		    );
		push(@error_loc, $text_line);
		last;
	    }
	}
    }
    show_loc();
    $main_window->configure(-cursor => $old_cursor);
    my $error_count = $#error_loc+1;
    $count_label->configure(-text => "$error_count Errors Marked");
}
########################################################
# ignore_cb -- Callback for the ignore button
#
# Add the excluded line to the list of lines we don't check and
# re-do the highlighting
########################################################
sub ignore_cb
{
    # The line number of the line we are ignoring
    my $line_number = $error_loc[$loc_index];
    # The line we are looking at
    my $line = $text_box->get(
	    "$line_number.0", "$line_number.0 lineend");
    # Quote all special characters
    $line =~ s/^ *\d+: //;
    # Escape      . [ ] * + \ ( )
    $line =~ s/([\.\[\]\*\+\\\(\)])/\\$1/g;
    push(@ignore_list, $line);
    
    $text_box->tagRemove('highlight', "0.0", "end");
    find_errors();
    $config_change = 1;
}
########################################################
# do_file($file_name) -- Process a file.
########################################################
sub do_file($)
{
    # The current file name
    my $file_name = shift;

    open IN_FILE, "<$file_name" or 
	die("Could not open input file: $file_name");

    # The main window for our GUI
    $main_window = MainWindow->new;
    $main_window->title("$file_name");
    $text_box = $main_window->Scrolled("Text")->pack(
	    -side => 'bottom',
	    -fill => 'both',
	    -expand => 1
	    );
    $text_box->configure(
	    -wrap => "none",
	    -background => "white",
	    -foreground => "black"
	);
    $text_box->tagConfigure('highlight',
	    -background => 'Red');

    $main_window->Button(
	    -text => 'First',
	    -command => sub {
		$loc_index = 0; 
		show_loc();
	    },
	    -background => "Cyan"
    )->pack(
	-side => 'left'
    );
    $main_window->Button(
	    -text => 'Next',
	    -command => sub {
	    	$loc_index++; 
		show_loc();
	    },
	    -background => "Cyan"
	)->pack(
	    -side => 'left'
	);
    $main_window->Button(
	    -text => 'Previous',
	    -command => sub {
	    	$loc_index--; 
		show_loc();
	    },
	    -background => "Cyan"
	)->pack(
	    -side => 'left'
	);
    $main_window->Button(
	    -text => 'Last',
	    -command => sub {
	    	$loc_index = $#error_loc; 
		show_loc();
	    },
	    -background => "Cyan"
	)->pack(
	    -side => 'left'
	);
    $count_label = $main_window->Label(
	    -text => "x Errors Marked"
	)->pack(
	    -side => 'left'
	);
    $main_window->Button(
	    -text => 'Exit',
	    -command => sub {
	    	write_config();
		exit(0);
	    },
	    -background => "red",
	    -foreground => "white"
	)->pack(
	    -side => 'right'
	);
    $main_window->Button(
	    -text => 'Ignore',
	    -command => \&ignore_cb,
	    -background => "grey",
	    -foreground => "black"
    )->pack(
	-side => 'right'
    );
    # Loop through numberering each line
    for (my $line_number = 1; ;$line_number++)
    {
        my $line = <IN_FILE>;
	if (!defined($line))
	{
	    last;
	}
	push(@file_lines, $line);
	# The line with line number
	my $full_line = sprintf "%4d: %s", $line_number, $line;
	$text_box->insert("end", $full_line);
    }
    close (IN_FILE);
    $text_box->configure(-state => 'disabled');
    find_errors();

    MainLoop();
}
my $config_name;		# name of the configuraiton file
########################################################
# Configuration file format
#
#	ERRORS
#	<error regular expression>
#	<error regular expression>
#	IGNORE
#	<ignore regular expression>
#	<ignore regular expression>
#	<ignore regular expression>
########################################################

########################################################
# read_config($config_name, $error_flag)
#
# Read the configuration file.
########################################################
sub read_config($$)
{
    # Set the name of the configuration file
    $config_name = shift;	

    # if true, exit on error
    my $error_flag = shift;	


    if (!open(CONFIG_FILE, "<$config_name"))
    {
	if ($error_flag)
	{
	    die("Could not open $config_name");
	}
	return;
    }

    @error_check = ();
    @ignore_list = ();

    while (<CONFIG_FILE>)
    {
	if (/^ERRORS$/)
	{
	    next;
	}
	if (/^IGNORE$/)
	{
	    last;
	}
	chomp($_);
	push(@error_check, $_);
    }
    while (<CONFIG_FILE>)
    {
	chomp($_);
	push(@ignore_list, $_);
    }
    close CONFIG_FILE;
}
########################################################
# write_config() -- Write the configuration file
########################################################
sub write_config()
{
    if (!$config_change)
    {
	return;
    }
    open CONFIG_FILE, ">$config_name"
	or die("Unable to open $config_name for writing");
    print CONFIG_FILE "ERRORS\n";
    foreach (@error_check)
    {
	print CONFIG_FILE "$_\n";
    }
    print CONFIG_FILE "IGNORE\n";
    foreach (@ignore_list)
    {
	print CONFIG_FILE "$_\n";
    }
    close CONFIG_FILE;
}

########################################################
########################################################
if (($#ARGV >= 0) &&  ($ARGV[0] eq "--help"))
{
    system("perldoc $0");
    exit;
}
my $status = getopts("c:");
if (!$status)
{
    usage();
}
if ($opt_c)
{
    read_config($opt_c, 1);
}
else
{
    read_config("$ENV{HOME}/.log_file", 0);
}

if ($#ARGV == -1)
{
    usage();
}
do_file($ARGV[0]);
