=pod

=head1 NAME

web-graph.pl - A program to produce a graph of the top ten web pages
on your server

=head1 SYNOPSIS

    perl web-graph.pl <log-file> [<log-file> ...]

=head1 DESCRIPTION

The I<web-graph.pl> analyzes your web logs (from the Apache
web server) and create a graph (in the file I<top_web.png>).

=head1 FILES

=over 4

=item top_web.png

The file in which the graph is written.

=back

=cut
use strict;
use warnings;
use GD::Graph::bars;
use web;

# Max number of items in the graph
use constant LIMIT => 9; 	

# Read each file in the list
foreach my $file (@ARGV) {
    read_info($file);
}
# The statics from the logs
my $stat_array = do_count();

# The sorted version of static
my @sort_array = sort {$b->{count} <=> $a->{count};}  @$stat_array;

# Limit the number of elements in the limit
if ($#sort_array > LIMIT) {
    @sort_array = @sort_array[0..LIMIT];
}

my $max = 0;	# The largest number of hits

# The data for plotting 
#	$raw_data[0] -- The page name
#	$raw_data[1] -- The counts
my @raw_data = ([], []);

# Build the data and at the same time
# compute the maximum
foreach my $cur_sort (@sort_array) {
    push(@{$raw_data[0]}, $cur_sort->{page});
    push(@{$raw_data[1]}, $cur_sort->{count});
    if ($cur_sort->{count} > $max) {
        $max = $cur_sort->{count};
    }
}
# Round the max up to the next 100
$max = int(($max + 100) / 100) * 100;

# Create the graph
my $my_graph = GD::Graph::bars->new(800, 600);

# Set the parameters
$my_graph->set( 
    x_label         => 'Web Page',
    y_label         => 'Hit Count',

    title           => 'Top Ten Web Pages',
    y_max_value     => $max,
    y_tick_number   => $max/100,
    y_label_skip    => 2,

    x_labels_vertical => 1,
    
    # shadows
    bar_spacing     => 8,
    shadow_depth    => 4,
    shadowclr       => 'dred',

    transparent     => 0,
) 
or warn $my_graph->error;

# Plot the data
$my_graph->plot(\@raw_data) or die $my_graph->error;

# Send the data to the file
open(OUT, ">top_web.png") or 
	die "Cannot open top-web.png for write: $!";
binmode OUT;
print OUT $my_graph->gd->png();
close OUT;
